#include <iostream>
#include <fstream>
#include <Eigen/Dense>
#include <Eigen/Eigenvalues>
#include <math.h>
#include <algorithm> // for copying
#include <locale>
#include "calculation.h"

#define PI 3.14159265

using namespace std;
using Eigen::MatrixXd;
using Eigen::EigenSolver;

template <class T>
T fromstring ( std::string s )
{
	T result;
	std::stringstream str;
	str << s;
	str >> result;
	return result;
}

class WithComma: public numpunct<char> // class for decimal numbers with comma
{
	protected: char do_decimal_point() const { return ','; } // override the function that gives the decimal separator
};


MatrixXd doCalculation(int input)
{

	MatrixXd test(2,2);

	for(int ti=0; ti<2; ti++){
		for(int tj=0; tj<2; tj++){
			test(ti,tj) =ti+tj;
		}
	}
/////////////////////////////////////////////////////////////////////////////////////////////
//////Dealing with the angles and creating a set of angles and planes that will be investigated////
///////////////////////////////////////////////////////////////////////////////////////////////////


double degreeInterval = 5.0; // starting from 0 to 180 with an interval of 5 degrees
double radianInterval = (PI/180)*degreeInterval;

int numbIntervals = ceil((PI+radianInterval)/radianInterval); //How many intervals??

MatrixXd intervalMatrix(1,numbIntervals); // the matrix which contains the radian values for each step
	intervalMatrix(0,0) = 0.0;
for(int i=0; i<numbIntervals-1; i++){
	intervalMatrix(0,i+1) = intervalMatrix(0,i) + radianInterval;
}

MatrixXd matrixTetta(1, numbIntervals*numbIntervals); //All Tetta angles
MatrixXd matrixPhi(1, numbIntervals*numbIntervals); //All phi angles

int k=0;
for(int i=0; i<numbIntervals*numbIntervals; i++){
	if(k==numbIntervals){
		k=0;
	}
	matrixTetta(0,i) = intervalMatrix(i/numbIntervals); // filling the matrix with tetta angles
	matrixPhi(0,i) = intervalMatrix(0,k); // filling the matrix with phi angles
	k++;
}

MatrixXd anglesMatrix(2,numbIntervals*numbIntervals); // (0,0...n) for Tettas, (1,0...n) for Phis

// Finally the wanted set of angles in anglesMatrix
for (int j=0; j<numbIntervals*numbIntervals; j++){
	anglesMatrix(0,j) = matrixTetta(0,j);
	anglesMatrix(1,j) = matrixPhi(0,j);

}


///////////////////////////////////////////////////
//// Dealing with the input in this section ///////
///////////////////////////////////////////////////



locale myloc(  locale(), new WithComma);// Own numeric facet
string readFromfile;
ifstream r_file_number("OperationTest.csv");
ifstream r_file("OperationTest.csv");
int numberoflines=0;
stringstream *temp; 
double n;

//How many lines are there in the input file to be read?
while (getline(r_file_number, readFromfile, ';')) {
    numberoflines++;
}
std::cout << "numberOfLines " << numberoflines << "\n" << std::endl;
std::cout << "(numberoflines-1)/6 " << (numberoflines-1)/6 << "\n" <<  std::endl;


int numbCols = 6; //one column for each stress component
int numbRows = ((numberoflines-1)/6); // on row for each stress tensor


// Puting all the stresshistories from the the excel file into a matrix where each row represents one stresstensor
// The first number on each line, the element number, in the csv-file is negelected
/* 
The format of inputData:
inputData(i,0) = sigmaX
inputData(i,1) = sigmaY
inputData(i,2) = sigmaZ
inputData(i,3) = tauXY
inputData(i,4) = tauXZ
inputData(i,5) = tauYZ
......
*/

MatrixXd inputData(numbRows,numbCols);
int count=0;
int numberOfLinesAfter=0;
while (getline(r_file, readFromfile, ';')) {
	if(numberOfLinesAfter == 0){ //Skipping the first line in every row
		numberOfLinesAfter++; 
		continue;
	}
	temp = new stringstream(readFromfile);
	temp -> imbue(myloc);
	temp->operator >> (n);
	inputData(numberOfLinesAfter-1, count) = n;
	count++;
	if(count==6){
		count=0;
		numberOfLinesAfter++;
	}
}


	// Just to make sure the format is correct... : printing to a file example.txt 
  ofstream myfile ("inputFile.txt");
  if (myfile.is_open())
  {
	for(int wR=0; wR<numbRows; wR++){
		for(int wC=0;wC<numbCols;wC++){
			myfile << inputData(wR,wC);
			myfile << "\n";
		}
		    myfile << "newline \n";
	}

    myfile.close();
  }

//////////////////////////////////////////////////////////////////////////////
///////// Dealing with the transformation matrix (a-values) //////////////////
//////////////////////////////////////////////////////////////////////////////

  	MatrixXd a11(1, numbIntervals*numbIntervals); // calculate the a values
	MatrixXd a12(1, numbIntervals*numbIntervals);
	MatrixXd a13(1, numbIntervals*numbIntervals);
	MatrixXd a21(1, numbIntervals*numbIntervals);
	MatrixXd a22(1, numbIntervals*numbIntervals);
	MatrixXd a23(1, numbIntervals*numbIntervals);
	MatrixXd a31(1, numbIntervals*numbIntervals);
	MatrixXd a32(1, numbIntervals*numbIntervals);
	MatrixXd a33(1, numbIntervals*numbIntervals);

	MatrixXd stressTensor(2,6);
	double sigmaXTransf, sigmaYTransf, sigmaZTransf, tauXYTransf, tauXZTransf, tauYZTransf;


	MatrixXd transformedStressTensor(2,6);
	double normalmax = 0.0;
	double skjerXYmax = 0.0;
	double skjerXYmin = 0.0;	
	double skjerviddeXY = 0.0;
	double skjerXZmax = 0.0;
	double skjerXZmin = 0.0;
	double skjerviddeXZ = 0.0;
	double resSkjervidde = 0.0;

	double findleyCriterion = 0.0;
	double findleyFactor=0.1362;

	double critTetta = 0.0;
	double critPhi = 0.0;
	double findleyCriterionLast = 0.0;
	double finalFindley = 0.0;
	double critnormalmax= 0.0;
	double critskjerXYmax = 0.0;
	double critskjerXYmin = 0.0;
	double critskjerXZmax = 0.0;
	double critskjerXZmin = 0.0;
	
	int nextline=0;
	int which=0;

	//findleys(numbRows, 4) isn't really used yet.. It's supposed to be a "sum up" of all the elements with critical findley values and angles.
	MatrixXd findleys(numbRows, 4);
	int progress =2;

	while(nextline < numbRows){ //For each element, not each stress tensor
	
		// find the 2*six next stresses and put them in two tensors   !!! OBS - not the general case - needs fixing
		for(int is=0; is<12; is++)
		{
			if(is<6){
			stressTensor(0,is)=inputData(nextline,is);
			}
			else{
				stressTensor(1,is-6)=inputData(nextline+1,is-6);
			}
		}

	//For each set of possible angles from the anglesMatrix, to calulcate the transformation matrix
	for(int ia=0; ia<numbIntervals*numbIntervals; ia++){

		a11(0,ia) = cos(anglesMatrix(0,ia))*sin(anglesMatrix(1,ia));
		a12(0,ia) = sin(anglesMatrix(0,ia))*sin(anglesMatrix(1,ia));
		a13(0,ia) = cos(anglesMatrix(1,ia));
		a21(0,ia) = -sin(anglesMatrix(0,ia));
		a22(0,ia) = cos(anglesMatrix(0,ia));
		a23(0,ia) = 0;
		a31(0,ia) = -cos(anglesMatrix(0,ia))*cos(anglesMatrix(1,ia));
		a32(0,ia) = -sin(anglesMatrix(0,ia))*cos(anglesMatrix(1,ia));
		a33(0,ia) = sin(anglesMatrix(1,ia));

		// for each stress history - !!!OBS not the general case - needs fixing
		for(int ie=0; ie<2; ie++)
		{

				transformedStressTensor(ie,0) = 
										a11(0,ia)*a11(0,ia)*stressTensor(ie,0) +
										a12(0,ia)*a12(0,ia)*stressTensor(ie,1) + 
										a13(0,ia)*a13(0,ia)*stressTensor(ie,2) +
										2*a11(0,ia)*a12(0,ia)*stressTensor(ie,3) +
										2*a11(0,ia)*a13(0,ia)*stressTensor(ie,4) +
										2*a13(0,ia)*a12(0,ia)*stressTensor(ie,5);
				transformedStressTensor(ie,1) =
										a21(0,ia)*a21(0,ia)*stressTensor(ie,0) +
										a22(0,ia)*a22(0,ia)*stressTensor(ie,1) + 
										a23(0,ia)*a23(0,ia)*stressTensor(ie,2) +
										2*a21(0,ia)*a22(0,ia)*stressTensor(ie,3) +
										2*a21(0,ia)*a23(0,ia)*stressTensor(ie,4) +
										2*a23(0,ia)*a22(0,ia)*stressTensor(ie,5);
				transformedStressTensor(ie,2) = 
										a31(0,ia)*a31(0,ia)*stressTensor(ie,0) +
										a32(0,ia)*a32(0,ia)*stressTensor(ie,1) + 
										a33(0,ia)*a33(0,ia)*stressTensor(ie,2) +
										2*a31(0,ia)*a32(0,ia)*stressTensor(ie,3) +
										2*a31(0,ia)*a33(0,ia)*stressTensor(ie,4) +
										2*a33(0,ia)*a32(0,ia)*stressTensor(ie,5);;
				transformedStressTensor(ie,3) = 
										a11(0,ia)*a21(0,ia)*stressTensor(ie,0) +
										a12(0,ia)*a22(0,ia)*stressTensor(ie,1) +
										a13(0,ia)*a23(0,ia)*stressTensor(ie,2) +
										(a11(0,ia)*a22(0,ia) + a12(0,ia)*a21(0,ia))*stressTensor(ie,3) +
										(a13(0,ia)*a21(0,ia) + a11(0,ia)*a23(0,ia))*stressTensor(ie,4) +
										(a12(0,ia)*a23(0,ia) + a13(0,ia)*a22(0,ia))*stressTensor(ie,5);
				transformedStressTensor(ie,4) =
										a11(0,ia)*a31(0,ia)*stressTensor(ie,0) +
										a12(0,ia)*a32(0,ia)*stressTensor(ie,1) +
										a13(0,ia)*a33(0,ia)*stressTensor(ie,2) +
										(a11(0,ia)*a32(0,ia) + a12(0,ia)*a31(0,ia))*stressTensor(ie,3) +
										(a13(0,ia)*a31(0,ia) + a11(0,ia)*a33(0,ia))*stressTensor(ie,4) +
										(a13(0,ia)*a32(0,ia) + a12(0,ia)*a33(0,ia))*stressTensor(ie,5);;
				transformedStressTensor(ie,5) =
										a21(0,ia)*a31(0,ia)*stressTensor(ie,0) +
										a22(0,ia)*a32(0,ia)*stressTensor(ie,1) +
										a23(0,ia)*a33(0,ia)*stressTensor(ie,2) +
										(a21(0,ia)*a32(0,ia) + a22(0,ia)*a31(0,ia))*stressTensor(ie,3) +
										(a23(0,ia)*a31(0,ia) + a21(0,ia)*a33(0,ia))*stressTensor(ie,4) +
										(a22(0,ia)*a33(0,ia) + a23(0,ia)*a32(0,ia))*stressTensor(ie,5);


				// just to make it a bit easier...
				sigmaXTransf = transformedStressTensor(ie,0);
				sigmaYTransf = transformedStressTensor(ie,1);
				sigmaZTransf = transformedStressTensor(ie,2);
				tauXYTransf = transformedStressTensor(ie,3);
				tauXZTransf = transformedStressTensor(ie,4);
				tauYZTransf = transformedStressTensor(ie,5);
			
					//Finding largest normal stress and shear stresses for the findley criterion calculation
					if(ie != 0)
					{
						if(sigmaXTransf > normalmax){
							normalmax = sigmaXTransf;
				
						}
							if(tauXYTransf > skjerXYmax){
							skjerXYmax = tauXYTransf;
						}
							if(tauXYTransf < skjerXYmin){
							skjerXYmin = tauXYTransf;
						}
							if(tauXZTransf > skjerXZmax){
							skjerXZmax = tauXZTransf;
						}
							if(tauXZTransf < skjerXZmin){
							skjerXZmin = tauXZTransf;
						}

					} else
					{
						normalmax= sigmaXTransf;
						skjerXYmax = tauXYTransf;
						skjerXYmin = tauXYTransf;
						skjerXZmax = tauXZTransf;
						skjerXZmin = tauXZTransf;
					}	

					// Finding max and min shear range
					skjerviddeXY = skjerXYmax-skjerXYmin;
					skjerviddeXZ = skjerXZmax-skjerXZmin;

					// The resulting shear range
					resSkjervidde = sqrt((skjerviddeXY*skjerviddeXY)+(skjerviddeXZ*skjerviddeXZ));

					// The findley criterion for the given element
					findleyCriterion = ((0.5*resSkjervidde)+(findleyFactor*normalmax));

					// Deciding which criterion is largest of the stress tensors for the element
					// Also finding the angles tetta and phi at the largest criterion
					if(ie != 0)
					{
						if(findleyCriterion > findleyCriterionLast)
						{
							critPhi = anglesMatrix(1,ia);
							critTetta = anglesMatrix(0,ia);
							finalFindley = findleyCriterion;
							normalmax= sigmaXTransf;
							critskjerXYmax = skjerXYmax;
							critskjerXYmin = skjerXYmin;
							critskjerXZmax = skjerXZmax;
							critskjerXZmin = skjerXZmin;
							which = nextline;

						}
					findleyCriterionLast = finalFindley;

					}
		}

		// This isn't really used...
			 findleys(nextline,0) = nextline;
			 findleys(nextline,1) = findleyCriterion;
			 findleys(nextline,2) = critTetta;
			 findleys(nextline,3) = critPhi;

	}
		nextline = nextline+2; // OBS!! Not the general case - needs some fixing

		/// This isn't really used either...
		if(nextline == progress*2){
			//std::cout << "nextline = " << nextline << std::endl;
			progress= progress + 2;
		}
}
	// At last! The largest findley criterion is found and the critical angles (and plane) with it. 
	std::cout << "Final Findley criterion = " << finalFindley << "\n" << std::endl;
	std::cout << "Final phi angle = " << critPhi*(180/PI) << "\n" << std::endl;

	std::cout << "Final tetta angle = " << critTetta*(180/PI) << "\n" << std::endl;
	std::cout << "Final element (line nr.. ) = " << which << "\n" << std::endl;

	MatrixXd crticalStressTensorAllAngles(numbIntervals*numbIntervals, 6);
		crticalStressTensorAllAngles = doAllCriticalTransformation(inputData, anglesMatrix, which,numbIntervals);


	std::cout << "Remsa = ferdig! \n" << std::endl;


	MatrixXd criticalStressTensor(3,3);
	MatrixXd transformedCriticalTensor(1,6); 

	transformedCriticalTensor = doTransformation(inputData, critTetta, critPhi, which);

	criticalStressTensor(0,0) = transformedCriticalTensor(0,0);
	criticalStressTensor(1,1) = transformedCriticalTensor(0,1);
	criticalStressTensor(2,2) = transformedCriticalTensor(0,2);
	criticalStressTensor(1,0) = transformedCriticalTensor(0,3);
	criticalStressTensor(0,1) = transformedCriticalTensor(0,3);
	criticalStressTensor(2,0) = transformedCriticalTensor(0,4);
	criticalStressTensor(0,2) = transformedCriticalTensor(0,4);
	criticalStressTensor(1,2) = transformedCriticalTensor(0,5);
	criticalStressTensor(2,1) = transformedCriticalTensor(0,5);


	EigenSolver<MatrixXd> es(criticalStressTensor);

	std::cout << "The eigenvalues of criticalStressTensor are:" << endl << es.eigenvalues() << endl;
	std::cout << "The matrix of eigenvectors, V, is:" << endl << es.eigenvectors() << endl << endl;



	std::cout << "Final element = \n" << criticalStressTensor << "\n" << std::endl;
	std::cout << "\n" << std::endl;

	 ofstream myfile2 ("Extra.txt");
  if (myfile2.is_open())
  {
	//  for(int wi=0; wi<crticalStressTensorAllAngles.innerSize(); wi++){
	//	  myfile2 << crticalStressTensorAllAngles(wi,0);
	//		myfile2 << "\n";


	//	}
	  myfile2 << "The Final Findley element: \n \n";
	  myfile2 << "PHI: "  << critPhi*(180/PI) << "\n";
	  myfile2 << "THETTA: " << critTetta*(180/PI) << "\n";
	  myfile2 << "The findley riterion Value: " << finalFindley << "\n \n";
	  myfile2 << "the tensor history 1:  \n" << inputData(which, 0) << " " << inputData(which, 1) << " " << inputData(which, 2) << " " << inputData(which, 3) << " " << inputData(which, 4) << " " << inputData(which, 5) << "\n";  
	  myfile2 << "the tensor histpry 2:  \n" << inputData(which+1, 0) << " " << inputData(which+1, 1) << " " << inputData(which+1, 2) << " " << inputData(which+1, 3) << " " << inputData(which+1, 4) << " " << inputData(which+1, 5) << "\n";
	  myfile2 << "the transformed stresses: \n";
	  myfile2 << "ONE: \n";
	  myfile2 << doTransformation(inputData, critTetta, critPhi, which) << "\n";
	  myfile2 << "TWO: \n";
	  myfile2 << doTransformation(inputData, critTetta, critPhi, which+1);
	  myfile2 << "critvalues \n: " ;
	  myfile2 << "critskjerXYmax = " << critskjerXYmax << "\n";
	  myfile2 << "critskjerXYmin = " << critskjerXYmin << "\n";
	  myfile2 << "critskjerXZmax = " << critskjerXZmax << "\n";
	  myfile2 << "critskjerXZmin = " << critskjerXZmin << "\n";
	  myfile2 << "skjervidde = " << sqrt((critskjerXYmax-critskjerXYmin)*(critskjerXYmax-critskjerXYmin) + (critskjerXZmax-critskjerXZmin)*(critskjerXZmax-critskjerXZmin)) << "\n";
	  myfile2 << "The crit = " << 0.5*sqrt((critskjerXYmax-critskjerXYmin)*(critskjerXYmax-critskjerXYmin) + (critskjerXZmax-critskjerXZmin)*(critskjerXZmax-critskjerXZmin))+findleyFactor*normalmax;
	  myfile2 << "\n" << "The principle stress matrix: " << es.eigenvalues();
	  myfile2 << "\n" << "The directions of the principle stress: " << es.eigenvectors();
	//myfile << "min: " << min << "\n";
	//myfile << "max: " << max << "\n";
	myfile2.close();
  }

	if(input == 1){
		return 	crticalStressTensorAllAngles;
	}
	else{
		return criticalStressTensor; // and eigenvalues... need to combine these into one matrix
	}

 // system("pause");
}

//
// doTransformation does the transformation of a stress tensor matrix (1,6) with given angles tetta and phi
// Used for transforming the critical element found and for the visualization part of the application
//

MatrixXd doTransformation(MatrixXd stressTensor, double tetta, double phi, int which){

MatrixXd transformedMatrix(1,6);

MatrixXd stressTensor2 (1, 6);

for(int i=0; i<6; i++){
	stressTensor2(0,i) = stressTensor(which, i);

}
	
double a11 = cos(tetta)*sin(phi);
double a12 = sin(tetta)*sin(phi);
double a13 = cos(phi);
double a21 = -sin(tetta);
double a22 = cos(tetta);
double a23 = 0;
double a31 = -cos(tetta)*cos(phi);
double a32 = -sin(tetta)*cos(phi);
double a33 = sin(phi);

transformedMatrix(0,0) = 
								a11*a11*stressTensor2(0,0) +
								a12*a12*stressTensor2(0,1) + 
								a13*a13*stressTensor2(0,2) +
								2*a11*a12*stressTensor2(0,3) +
								2*a11*a13*stressTensor2(0,4) +
								2*a13*a12*stressTensor2(0,5);
transformedMatrix(0,1) =
								a21*a21*stressTensor2(0,0) +
								a22*a22*stressTensor2(0,1) + 
								a23*a23*stressTensor2(0,2) +
								2*a21*a22*stressTensor2(0,3) +
								2*a21*a23*stressTensor2(0,4) +
								2*a23*a22*stressTensor2(0,5);
transformedMatrix(0,2) = 
								a31*a31*stressTensor2(0,0) +
								a32*a32*stressTensor2(0,1) + 
								a33*a33*stressTensor2(0,2) +
								2*a31*a32*stressTensor2(0,3) +
								2*a31*a33*stressTensor2(0,4) +
								2*a33*a32*stressTensor2(0,5);;
transformedMatrix(0,3) = 
								a11*a21*stressTensor2(0,0) +
								a12*a22*stressTensor2(0,1) +
								a13*a23*stressTensor2(0,2) +
								(a11*a22 + a12*a21)*stressTensor2(0,3) +
								(a13*a21 + a11*a23)*stressTensor2(0,4) +
								(a12*a23 + a13*a22)*stressTensor2(0,5);
transformedMatrix(0,4) =
								a11*a31*stressTensor2(0,0) +
								a12*a32*stressTensor2(0,1) +
								a13*a33*stressTensor2(0,2) +
								(a11*a32 + a12*a31)*stressTensor2(0,3) +
								(a13*a31 + a11*a33)*stressTensor2(0,4) +
								(a32*a32 + a12*a33)*stressTensor2(0,5);;
transformedMatrix(0,5) =
								a21*a31*stressTensor2(0,0) +
								a22*a32*stressTensor2(0,1) +
								a23*a33*stressTensor2(0,2) +
								(a21*a32 + a22*a31)*stressTensor2(0,3) +
								(a23*a31 + a21*a33)*stressTensor2(0,4) +
								(a22*a33 + a23*a32)*stressTensor2(0,5);


	return transformedMatrix;
}



MatrixXd doAllCriticalTransformation(MatrixXd stressTensor, MatrixXd anglesMatrix, int which, int numbIntervals){

	MatrixXd transformedStressTensor(numbIntervals*numbIntervals,6);

	MatrixXd a11(1, numbIntervals*numbIntervals); // calculate the a values
	MatrixXd a12(1, numbIntervals*numbIntervals);
	MatrixXd a13(1, numbIntervals*numbIntervals);
	MatrixXd a21(1, numbIntervals*numbIntervals);
	MatrixXd a22(1, numbIntervals*numbIntervals);
	MatrixXd a23(1, numbIntervals*numbIntervals);
	MatrixXd a31(1, numbIntervals*numbIntervals);
	MatrixXd a32(1, numbIntervals*numbIntervals);
	MatrixXd a33(1, numbIntervals*numbIntervals);

		for(int ia=0; ia<numbIntervals*numbIntervals; ia++){

			a11(0,ia) = cos(anglesMatrix(0,ia))*sin(anglesMatrix(1,ia));
			a12(0,ia) = sin(anglesMatrix(0,ia))*sin(anglesMatrix(1,ia));
			a13(0,ia) = cos(anglesMatrix(1,ia));
			a21(0,ia) = -sin(anglesMatrix(0,ia));
			a22(0,ia) = cos(anglesMatrix(0,ia));
			a23(0,ia) = 0;
			a31(0,ia) = -cos(anglesMatrix(0,ia))*cos(anglesMatrix(1,ia));
			a32(0,ia) = -sin(anglesMatrix(0,ia))*cos(anglesMatrix(1,ia));
			a33(0,ia) = sin(anglesMatrix(1,ia));

				transformedStressTensor(ia,0) = 
										a11(0,ia)*a11(0,ia)*stressTensor(which,0) +
										a12(0,ia)*a12(0,ia)*stressTensor(which,1) + 
										a13(0,ia)*a13(0,ia)*stressTensor(which,2) +
										2*a11(0,ia)*a12(0,ia)*stressTensor(which,3) +
										2*a11(0,ia)*a13(0,ia)*stressTensor(which,4) +
										2*a13(0,ia)*a12(0,ia)*stressTensor(which,5);
				transformedStressTensor(ia,1) =
										a21(0,ia)*a21(0,ia)*stressTensor(which,0) +
										a22(0,ia)*a22(0,ia)*stressTensor(which,1) + 
										a23(0,ia)*a23(0,ia)*stressTensor(which,2) +
										2*a21(0,ia)*a22(0,ia)*stressTensor(which,3) +
										2*a21(0,ia)*a23(0,ia)*stressTensor(which,4) +
										2*a23(0,ia)*a22(0,ia)*stressTensor(which,5);
				transformedStressTensor(ia,2) = 
										a31(0,ia)*a31(0,ia)*stressTensor(which,0) +
										a32(0,ia)*a32(0,ia)*stressTensor(which,1) + 
										a33(0,ia)*a33(0,ia)*stressTensor(which,2) +
										2*a31(0,ia)*a32(0,ia)*stressTensor(which,3) +
										2*a31(0,ia)*a33(0,ia)*stressTensor(which,4) +
										2*a33(0,ia)*a32(0,ia)*stressTensor(which,5);;
				transformedStressTensor(ia,3) = 
										a11(0,ia)*a21(0,ia)*stressTensor(which,0) +
										a12(0,ia)*a22(0,ia)*stressTensor(which,1) +
										a13(0,ia)*a23(0,ia)*stressTensor(which,2) +
										(a11(0,ia)*a22(0,ia) + a12(0,ia)*a21(0,ia))*stressTensor(which,3) +
										(a13(0,ia)*a21(0,ia) + a11(0,ia)*a23(0,ia))*stressTensor(which,4) +
										(a12(0,ia)*a23(0,ia) + a13(0,ia)*a22(0,ia))*stressTensor(which,5);
				transformedStressTensor(ia,4) =
										a11(0,ia)*a31(0,ia)*stressTensor(which,0) +
										a12(0,ia)*a32(0,ia)*stressTensor(which,1) +
										a13(0,ia)*a33(0,ia)*stressTensor(which,2) +
										(a11(0,ia)*a32(0,ia) + a12(0,ia)*a31(0,ia))*stressTensor(which,3) +
										(a13(0,ia)*a31(0,ia) + a11(0,ia)*a33(0,ia))*stressTensor(which,4) +
										(a13(0,ia)*a32(0,ia) + a12(0,ia)*a33(0,ia))*stressTensor(which,5);;
				transformedStressTensor(ia,5) =
										a21(0,ia)*a31(0,ia)*stressTensor(which,0) +
										a22(0,ia)*a32(0,ia)*stressTensor(which,1) +
										a23(0,ia)*a33(0,ia)*stressTensor(which,2) +
										(a21(0,ia)*a32(0,ia) + a22(0,ia)*a31(0,ia))*stressTensor(which,3) +
										(a23(0,ia)*a31(0,ia) + a21(0,ia)*a33(0,ia))*stressTensor(which,4) +
										(a22(0,ia)*a33(0,ia) + a23(0,ia)*a32(0,ia))*stressTensor(which,5);


		}

		return transformedStressTensor;
}



////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////**Pensjonert Kode**/////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////
/*


/////
//////// Code for solving a transformation of a matrix with angles tetta and phi
////

// angles for transformation
int tettaDeg = 10; 
int phiDeg = 20;

//Degrees to radians
double tetta = tettaDeg*PI/180;
double phi = phiDeg*PI/180;

// Constants for transformation matrix
double a11 = cos(tetta)*sin(phi);
double a12 = sin(tetta)*sin(phi);
double a13 = cos(phi);
double a21 = -sin(tetta);
double a22 = cos(tetta);
double a23 = 0;
double a31 = -cos(tetta)*cos(phi);
double a32 = -sin(tetta)*cos(phi);
double a33 = sin(phi);

std::cout << "a11 = "  << a11 << "\n" << std::endl;
std::cout << "a12 = "  << a12 << "\n" << std::endl;
std::cout << "a13 = "  << a13 << "\n" << std::endl;
std::cout << "a21 = "  << a21 << "\n" << std::endl;
std::cout << "a22 = "  << a22 << "\n" << std::endl;
std::cout << "a23 = "  << a23 << "\n" << std::endl;
std::cout << "a31 = "  << a31 << "\n" << std::endl;
std::cout << "a32 = "  << a32 << "\n" << std::endl;
std::cout << "a33 = "  << a33 << "\n" << std::endl;

//The stress tensor to be transformed

stressTensor(0,0) = 30; 
stressTensor(0,1) = 60;
stressTensor(0,2) = 0;
stressTensor(0,3) = 10;
stressTensor(0,4) = 0;
stressTensor(0,5) = 0;

	
double sigmaXTransf, sigmaYTransf, sigmaZTransf, tauXYTransf, tauXZTransf, tauYXTransf;

sigmaXTransf = transformedStressTensor(0,0);
sigmaYTransf = transformedStressTensor(0,1);
sigmaZTransf = transformedStressTensor(0,2);
tauXYTransf = transformedStressTensor(0,3);
tauXZTransf = transformedStressTensor(0,4);
tauYZTransf = transformedStressTensor(0,5);


MatrixXd transformedStressTensor(1,6);
transformedStressTensor(0,0) = 
								a11*a11*stressTensor(0,0) +
								a12*a12*stressTensor(0,1) + 
								a13*a13*stressTensor(0,2) +
								2*a11*a12*stressTensor(0,3) +
								2*a11*a13*stressTensor(0,4) +
								2*a13*a12*stressTensor(0,5);
transformedStressTensor(0,1) =
								a21*a21*stressTensor(0,0) +
								a22*a22*stressTensor(0,1) + 
								a23*a23*stressTensor(0,2) +
								2*a21*a22*stressTensor(0,3) +
								2*a21*a23*stressTensor(0,4) +
								2*a23*a22*stressTensor(0,5);
transformedStressTensor(0,2) = 
								a31*a31*stressTensor(0,0) +
								a32*a32*stressTensor(0,1) + 
								a33*a33*stressTensor(0,2) +
								2*a31*a32*stressTensor(0,3) +
								2*a31*a33*stressTensor(0,4) +
								2*a33*a32*stressTensor(0,5);;
transformedStressTensor(0,3) = 
								a11*a21*stressTensor(0,0) +
								a12*a22*stressTensor(0,1) +
								a13*a23*stressTensor(0,2) +
								(a11*a22 + a12*a21)*stressTensor(0,3) +
								(a13*a21 + a11*a23)*stressTensor(0,4) +
								(a12*a23 + a13*a22)*stressTensor(0,5);
transformedStressTensor(0,4) =
								a11*a31*stressTensor(0,0) +
								a12*a32*stressTensor(0,1) +
								a13*a33*stressTensor(0,2) +
								(a11*a32 + a12*a31)*stressTensor(0,3) +
								(a13*a31 + a11*a33)*stressTensor(0,4) +
								(a12*a32 + a12*a33)*stressTensor(0,5);;
transformedStressTensor(0,5) =
								a21*a31*stressTensor(0,0) +
								a32*a32*stressTensor(0,1) +
								a23*a33*stressTensor(0,2) +
								(a21*a32 + a22*a31)*stressTensor(0,3) +
								(a23*a31 + a21*a33)*stressTensor(0,4) +
								(a22*a33 + a23*a32)*stressTensor(0,5);







std::cout << "\nThe transformed stress tensor: " << std::endl;
std::cout << transformedStressTensor << std::endl;

*/

/*
	// Just to make sure the format is correct... : printing to a file example.txt 
  ofstream myfile ("example.txt");
  if (myfile.is_open())
  {
	for(int wR=0; wR<numbRows; wR++){
		for(int wC=0;wC<numbCols;wC++){
			myfile << inputData(wR,wC);
			myfile << "\n";
		}
		    myfile << "newline \n";
	}

    myfile.close();
  }

// double numbStories = numberoflines/numberOfLinesAfter; TESINGTESING
// std::cout << "numberoflines = " << numberoflines << "\n" << std::endl;
// std::cout << "numberOfLinesAfter = " << numberOfLinesAfter << "\n" << std::endl;
// std::cout << "numbStories should be 2 = " << numbStories << "\n" << std::endl;
*/